
/************************************************************************************
	
	Spektralregler fr Pulsschweimaschinen
	ionOS		Ion Operating System for AVR Xmega
	PCB			ION_v11
	Created		23/04/2013
	Compilation 27/09/2016
	Autor		heinz@gfai.de
	AVR-Studio4	GUI Version 4,19,0,730

*/ 	char version[48] = " ionOSv11 www.gfai.de/~heinz CV 27/09/2019 "; /*

	Alle Rechte: GFaI e.V.

	Mitarbeiter:
		Spektralregler ION & ionOS:	Gerd Heinz
		Windows-Oberflche Lion:	Axel Groehling

	Hotline: hwis@gfai.de

	Prozessor enstellen - ATXmega192A3 bzw. ATXmega64A3
		Project > Configuration Options > Active Configuration

	Funkktionsbeschreibung siehe Datenblatt_Spektralregler.txt	
	Reglerfunktion ndern? Siehe control.c  oder suche nach: "Regler("

	Vorsicht: Eventsystem wurde teils wieder abgeschaltet, siehe Marken #####
	Eventsystem beharkte sich mit Interrupts (stotternder Systick-Interrupt)
	Evtl. sind noch ungltige Kommentare verblieben.

	Set ATxmega64a3/192a3 Fuses:
		FUSEBYTE0	ff
		FUSEBYTE1	aa
		FUSEBYTE2	7f
		FUSEBYTE4	f6
		FUSEBYTE5	ff

*************************************************************************************/

// Core:
#define F_CPU 32000000UL		// Taktfrequenz mu vor avr_compiler.h stehen
// Prozessor ist nur im Projektfile einzustellen: Project -> "Configuration Options"
// #define __AVR_ATxmega192A3__	//../ionOS.c:49:0: warning: "__AVR_ATxmega192A3__" redefined
// #define __AVR_ATxmega64A3__	//../ionOS.c:49:0: warning: "__AVR_ATxmega64A3__" redefined

#include "xmega/avr_compiler.h"
// bindet #include <avr/io.h> ein
/* mit <avr/io.h> werden geladen:
	// ioxxxx.h 				// die Prozessorregister
	#include <avr/sfr_defs.h>
	#include <avr/portpins.h>	// man knnte dann "PD3" schreiben
	#include <avr/common.h>
	#include <avr/version.h>
	#include <avr/fuse.h>
	#include <avr/lock.h>
*/

// Functions
#include <stdint.h>				// uint8_t etc. Std-Integers
#include <string.h>				// memcpy
#include <util/atomic.h>		// cli, sei, ISR ...
//#include <stdio.h>			// printf scanf etc.
//#include <stddef.h>			// Typen
//#include <math.h>				// FPU

// AVR:
//#include <avr/signature.h>	// check signature of device before programming
//#include <avr/pgmspace.h>		// strings in Flash
//#include <avr/interrupt.h>	// definiert cli() und sei()

// Project specific
#include "src/global.h"			// prototypes and structure definitions
#include "src/control.h"		// Reset, Watchdog, Weld, Regler, Menus and Dialogs

// Peripheral blocks
#include "src/eeprom.h"			// 
#include "src/adc.h"			// 
#include "src/button.h"			// 
#include "src/clocks.h"			// oscillator and timer
#include "src/dac.h"			// 
#include "src/leds.h"			// 
#include "src/spi.h"			// 
#include "src/uart.h"			//
// #include "src/mram.h"		// 

// globale Variablen aus global.h
char error[48] = " ionOS hat Befehl nicht erkannt ";
volatile enum 	sig_t 		bufsig;	// RX buffer control
volatile enum 	com_t 		comreg;	// UART/SPI Register
volatile enum 	mode_t 		mode;	// wld, com, rec
volatile enum 	cmd_t 		cmd;	// Befehle d, i, r,...
volatile enum 	led_t 		leds;	// LEDs
reg_t 		REG;	// UART/SPI register
rec_t 		REC;	// record frame
par_t 		PRL;	// parameter list
frame_t		FR;		// communication-frame

// hier sind benutzte Regler-Funktionen aufzulisten (max. 32000):
VoidFct Regler[] = {R0, R1, R2, R3, R4, R5, R6, R7, R8, R9};
	// Aufruf z.B.: Regler[1](); // ruft R1();


/***********************************************************************************/
/*** main **************************************************************************/
/***********************************************************************************/

int main( void )			// 
{ 	// boot the device
	cli();					// Interrupts sperren
	disableTimer();			// abwarten
	watchdog_config();		// init watchdog

	LEDs_config();			// config LEDs 
	BTN_config();			// config ENTER-Button

	display(red);			// show any reset

	CLK_config();			// XOSC & PLL init oscillator and system clocks
	// CPUclk_setpin();		// show CPU-clock at testpin // Pin wurde anders belegt

	Systick_config();		// SysTick-Timer config
	Systick_setpin();		// show SysTick at testpin

	Beat_config();			// HeartBeat timer
	EVS_config();			// config event system SysTick, HeartBeat
	PMIC_init();			// Programmable Interrupt Controller freigeben

	EEPROM_Init();			
	EEPROMreadParlist();	// Parameterliste aus EEPROM lesen fr Potis, DAC...

	DAC_config();			// reference voltages ARA and ARB 1,65V
	ADC_config();			// analog inputs K1...K6, KU, KI

	RECinit();				// REC-struct init (framesync G)

	SPI_POTIconfig();		// Ports fr Potis einstellen
	initPotis();			// Potis ber SPI einstellen

	SCI_UART_config();		// ABIC init SCI-UART 
	SCI_disable();			// SSC-auto-mode herstellen (SPI-Shiftregister SSC)

	SSC_SPI_config();		// ABIC Shiftregister SPI fr ABIC-SSC

	RS485_config();			// dRec data recorder interface als RS485 duplex
	bypass_config();		// fast bypass optocoupler
	
	// display(green);			// toggle LEDs
	toggleTickpin();		// toggle systick-pin

	//////////////////////////////////////////////////////////////////////////

	// Factory-Reset mit ENTER-Taste: 
	if ( !( BTNPORT.IN & BTNPIN ) )	{ 			// ENTER-Taste
		// wenn ENTER bei RESET gedrckt war (PC0 = 0):
		// Generiert Parlist PRL, ldt Parlist to eeprom, schaltet wld-Mode ein
		genPar();			// Parametersatz generieren, incl. Potiwerte
		initPotis();		// Potis ber SPI einstellen
		toggleTickpin(); 
		display(red);
		EEPROMsaveParlist();	// Parameterliste sichern
		toggleTickpin(); 
		// display(green);
	}
	else {		// ENTER war nicht gedrckt: Schweien mit letztem Parametersatz
		// display(off);		// Mode anzeigen
	}

	setComPort(rs);				// default port
	//sendParlist();			// Parliste an PC senden (nur zum Test)
	bufsig = ready;				// initial
	prepareWeld();				// includes sei(), enableTimer... 

	//////////////////////////////////////////////////////////////////////////

	while(1) {					// wait for interrupts

		// Sind Daten angekommen als Frame FR ?
		if (bufsig == string_complete) {
			bufsig = ready;		// bufsig rcksetzen
			mode = com;			// Betriebsart umschalten auf Kommunikation
			// display(red);		// nur frs debuging
			cmdDecode(FR.C);	// Befehl decodieren und ausfhren
		}
	toggleTickpin();		// test occurence in slope
	asm volatile ("nop");	// Compiler soll die Schleife nicht wegoptimieren
	}
return 0;
}


/***********************************************************************************/
/*** Interrupts ********************************************************************/
/***********************************************************************************/


// Herzschlag *****************************************************************
ISR(TCC1_OVF_vect) 	{		// Timer TCC1-overflow generated heartbeat
	// display(toggle); 			// change LEDs state every half second
}


// Daten senden ////////////////////////////////////////////////////////////////////

// SysTick-Timer Interrupt *********************************************************
ISR(TCC0_OVF_vect) 	{		// Timer TCC0-overflow generates systick 0,1 ms ~ 10 kHz
	toggleTickpin();		// Marke fr den Logikanalyzer
	// display(off);			// Beginn anzeigen
	//pop();
	switch (mode) {
		case rec: {				// ADC in freerun 500 kHz 
				Regler[PRL.NA](); // ADCs lesen, Regelgren berechnen, SM steuern mit BYPASS
				sendRec();		// Ergebnisse der letzten Regelung zu Lion versenden
				display(green);	// Ende - Marke fr den Logikanalyzer
				break;}
		case wld: { 			// ADC in freerun 500 kHz 
				Regler[PRL.NA](); // ADCs lesen, Regelgren berechnen, SM steuern mit BYPASS
				display(blue);	// Ende - Marke fr den Logikanalyzer
				break;}
		case com: {
				disableTimer();	// Timer-Interrupts systick + heartbeat abschalten
				display(red);	// Beginn anzeigen
				break;}
		default: { 
				mode = wld;
				enableTimer();
				break;}
	}
	//push();
	watchdog_reset();		// Watchdog rcksetzen, sonst time-out reset
}


// Daten empfangen ////////////////////////////////////////////////////////////

// RS485 UART Interrupts, Port:C0 *********************************************
ISR(USARTC0_RXC_vect)	{ 	// RX Receive Complete Interrupt ###
	disableTimer();	
	mode = com;				// weld off, rec off, communication on
	//setComPort(rs);		// kostet zuviele Takte
	getx( (uint8_t *) &FR, 128);	// cash incomming string
}

// ABIC-SCI UART Interrupts, Port:E0 ******************************************
ISR(USARTE0_RXC_vect)	{ 	// RX Receive Complete Interrupt ###
	disableTimer();	
	mode = com;				// weld off, rec off, communication on
	setComPort(sci);
	getx( (uint8_t *) &FR, 128);	// cash incomming string
}

// ABIC-SSC SPI-Interrupts, Port:E ********************************************
ISR(SPIE_INT_vect)	{		// SPI-Interrupt ###
	disableTimer();	
	mode = com;				// weld off, rec off, communication on
	setComPort(ssc);
	getx( (uint8_t *) &FR, 128);	// cash incomming string
}


