#ifndef ADC_H
#define ADC_H



// ADC ***************************************************

/* 	Wird interne Referenz REFSEL[4:3] = 00 (1.00V) genommen, 
ist max. Output 0.98 Volt. Wird REFSEL[4:3] = 01 (AVCC) genommen, kommen 
wir auf 3.1 Volt; jeweils bei Vollaussteuerung mit 0x0FFF ~ 4095 dez. 
Deshalb nehmen wir externe Referenz 1,65V am ADC_PA0: ARA
*/


// ADCA Pins (PORTA fixed) A-Manual, p.290
#define CHA0PIN PIN2		// K1
#define CHA1PIN PIN3		// K2
#define CHA2PIN PIN4		// K3
#define CHA3PIN PIN5		// K4

// ADCB Pins (PORTB fixed) A-Manual, p.290
#define CHB0PIN PIN4		// K5
#define CHB1PIN PIN5		// K6
#define CHB2PIN PIN6		// KU
#define CHB3PIN PIN7		// KI


// Vorteiler ADC_CLK, 32MHz/64 = 500kHz 
// -> je 1/(500kHz/8) = 16 s + 3x2s = 22s for all chls, p.298
#define ADCxPRESCAL 0x04			// 100: Vorteiler /64

// ADCxEVCTRL = 11,111,110: SWEEP,EVSEL,EVACT four channel syncsweep on evchl 7,	p.304
		// SWEEP = 11: 	adc channel selection: 		ADC-channels 0,1,2,3
		// EVSEL = 111: event input select: 		event source is evchl 7
		// EVACT = 110:	adc event mode select:		syncsweep on all channels
//#define ADCxEVCTRL  0b11111110		// 11 111 110 four channel sweep on evtchl 7	#####
//#define ADCxEVCTRL  0x00			// no event control -> set freerun in ADCxCTRLB
//#define ADCxEVCTRL  0b11000000		// no event control, select all channels, freerun
#define ADCxEVCTRL  0b11000110		// 11 000 110 four channel sweep on all eventchls

#define ADCxCTRLB	0b00011000		// signed, freerun, 12bit right adj
//#define ADCxCTRLB 0b00010000		// signed, no freerun, 12 bit right adjusted
//#define ADCxCTRLB	0b00011110		// signed, freerun, 12bit left adj
//#define ADCxCTRLB	0b00010110		// signed, no freerun, 12bit left adj		#####

// external reference external from AREFA pin on PORTA (ARA)
//#define ADCxREFCTRL 0b00100010	// VREF = ARA, BANDGAP on for brown-out detection
#define ADCxREFCTRL 0b00100000		// VREF = ARA (ADCA and ADCB)

// ADC enable
//#define ADCxCTRLA_DMAreq 0b11000001	// DMA request, ADC ENABLE
#define ADCxCTRLA 	0b00111101		// CH3...CH0 StartSingleConv; ADC ENABLE
//#define ADCxCTRLA 0b00000001		// ADC ENABLE	#####

// ADC conversion complete interrupt
#define ADCxINTCTRL	0x00			// interrupts disabled (default)

// Pinzuordnung differential with ADC-PIN0 (PA0:ARA, PB0:ARB):
#define ADCxMUXCTRL 0x00			// p.310, MUXNEG = 00: PIN0 ~ AREFA, AREFB, Pinzuordnung mit CHxyPIN

// ADC channel control register
#define ADCxCHyCTRL 0b10000010		// 1 00 000 10 start, differential p.310


void ADC_config(void);


#endif ////////////////////////// end of header /////////////////////////////////
