#include "../xmega/avr_compiler.h"		// ldt Pins und Ports
#include "../xmega/clksys_driver.h"	// avr-driver for clocks and oscillator

#include "global.h"
#include "clocks.h"
#include "leds.h"



// Event System *******************************************************************
// abgeschaltet --> stottert


// Timer stt ADC an
void EVS_config()	{	// CHxMUX, A-Manual, p.75	
	// SysTick verteilen auf dem Event System Kanal 7
	// Event source: Timer C0 Overflow (TCC0_OVF) 
//	EVSYS.CH7MUX = EventSource_TCC0_OVF;	// 1100 0 000, A-Manual, p.72	#####
}


/**************************************************************/
// Set CPU-Clock-Testpin 
/* war nur Test
void CPUclk_setpin(void) {			// event chl.0 output p.145, tab.13-7
	CLK_PORT.DIR |= (1<<PIN7);		// set direction to output	###
	PORTCFG.CLKEVOUT |= CLK_EV;		// siehe Tab.13-7,p.145
}
*/

/**************************************************************/
// Set SysTick testpin 
/*
#define TICKPORT PORTF		// systick test output port		###
#define TICKPIN (1<<PIN6)	// pin number for systick 		###
*/

void Systick_setpin() {			// systick auf diesem Pin ausgeben
	// toggle testpin
	TICKPORT.DIR |= TICKPIN;		// dir = output
}


void toggleTickpin() {				// systick auf diesem Pin ausgeben
	// toggle testpin
	TICKPORT.OUT ^= TICKPIN;		// EXOR toggle systick
}

/**************************************************************/
// Set Clocksystem and Oscillator Control *********************
/**************************************************************/

// Clock System		XMEGA-A-manual p.77
// Oscillator Control	XMEGA-A-manual p.87

// INPUTs
// 		Quartz	8 MHz
//		intRC	2 MHz (boot only)

// PLL: 		8 MHz * 8 = 64 MHz			PLLCTRL.PLLFAC = 8, p.90
// Vorteiler: 	PSCTRL.PSBCDIV = 1, CLKPER = CLKPLL/2 = 32 MHz, p. 80

// PLL:			clksys = 10 * 8MHz = 80MHz: PLLCTRL.PLLFAC = 10; p.90
// Vorteiler: 	PSCTRL.PSBCDIV = 1, CLKPER = CLKPLL/2 = 40 MHz, p. 80

// OUTPUTs				  FACTOR	CLKs			USE
// 		after Prescaler A 	/1		clkper4			---
//		after Prescaler B	/1		clkper2			---
//		after Prescaler C 	/2		clkper, clkcpu	cpu, uart, ram


// siehe Clock System und Oscillator Control, p.80, p.91
void CLK_config(void)	{	// mit clksys_driver (!)	p.77, p.87
	// built system clock 32MHz with Quarz XOSC 8MHz, 
	// PLL Faktor 4: 64 MHz. After reset 
	// is the internal 2MHz RC-oscillator 
	// (System Clock Selection: RC2MHz) active.
	// all enums see iox192a3.h
	
	// Oszillator XOSC konfigurieren (8 MHz, 256 clocks startup time)
	CLKSYS_XOSC_Config( OSC_FRQRANGE_2TO9_gc, false, OSC_XOSCSEL_XTAL_256CLK_gc );

	// Oszillator XOSC enable
	CLKSYS_Enable( OSC_XOSCEN_bm );

	// PLLCTRL.PLLFAC p.90
		// 	clksys =  4 * 8MHz = 32MHz: PLLFACTOR = 0x04;
		// 	clksys =  8 * 8MHz = 64MHz: PLLFACTOR = 0x08;
		// 	clksys = 10 * 8MHz = 80MHz: PLLFACTOR = 0x0a;
	CLKSYS_PLL_Config( OSC_PLLSRC_XOSC_gc, PLLFACTOR );

	// Warten bis der Oszillator bereit ist
	do  {display(red); } 
	while ( CLKSYS_IsReady( OSC_XOSCRDY_bm ) == 0 );
	display(off);

	// Enable PLL
	CLKSYS_Enable( OSC_PLLEN_bm );

	// Prescalers division by A=1, B=1, C=2
	// 		clkper4 = clkper2  = 64 MHz, clkper = clkcpu = 32 MHz  (UART, CPU)
	// CLKSYS_Prescalers_Config( CLK_PSADIV_1_gc, CLK_PSBCDIV_1_2_gc );	// PLL 64MHz
	// Prescalers division by A=1, B=1, C=1
	CLKSYS_Prescalers_Config( CLK_PSADIV_1_gc, CLK_PSBCDIV_1_1_gc );	// PLL 32MHz

	// wait until PLL is locked
	do { display(red); } 
	while ( CLKSYS_IsReady( OSC_PLLRDY_bm ) == 0 );
	display(green);

	// Main Clock Source is PLL
	CLKSYS_Main_ClockSource_Select( CLK_SCLKSEL_PLL_gc );
	// Nun luft der Xmega Takt auf 32MHz
};


/***********************************************************************/
// TIMER ***************************************************************
/***********************************************************************/

// Systick Timer TCC0 *********************************************
	// input: ClkCPU
	// output: SysTick: as ovf-interrupt and on event chl 7
	// Event Channel 0	SYSTICK on TCC0_OVF_vect

void Systick_config(void) { 	// TCC0_OVF_vect   Sample-Takt einstellen 
	cli();				// Interrupts sperren, der Timer luft bereits! (Atmel-Scherz)

	PRL.FT = F_TICK;	// read only (uint16_t)
	TCC0.PER = PRL.FT;	// Timer-Endwert setzen fr SysTick

	TCC0.CTRLA = TCC_EVENT_inpClk; 	// CLKSEL[3:0] = 0 001, ohne Vorteiler, Source ist clkper
	TCC0.CTRLB = 0x00; 				// kein CompCapture, keine WaveGen
	TCC0.CTRLD = TCC_EVENT_out7;	// EVACT,EVDLY,EVSEL: 100 0 1111	p.167	nicht freigegeben #####
	//TCC0.CTRLD = TCC_EVENT_noOut;	// EVACT,EVDLY,EVSEL: 100 0 0000	p.167
	// EVACT[7:5] = 100 event action: restart counter
	// EVDLY[4] = 1	event delay for cascading counters
	// EVSEL[3:0] = 1111 event source select 1xxx, 111: event channel 7 is source for ADC and TCC1
	TCC0.INTCTRLA = TIMER_OVF_INT_medium; 	// Interrupt level medium, p.125
}


// Heartbeat Timer TCC1 *******************************************
	// input:	event-channel 7 (SysTick)
	// output: 	half-second clock as ovf-interrupt
	// als Timer C1 Overflow (mit TCC1_OVF_vect)

void Beat_config(void) {	// TCC1_OVF_vect
	cli();					// Interrupts sperren, der Timer luft immer (Atmel-Gag)

	// Heartbeat generieren auf Timer/Counter TCC1 aus systick
	TCC1.PER = F_BEAT;		// Anzahl der Ticks pro Heartbeat; read only (uint16_t)

	//TCC1.CTRLA = TCC_EVENT_inp7;	// CLKSEL[3:0] = 1 111 (event channel 7 as source) #####
	TCC1.CTRLA = TCC_CLKSRC_1024;	// CLKSEL[3:0] = 0 111 (clk/1024 as source)
	TCC1.CTRLB = 0x00; 				// kein CompCapture, keine WaveGen
	TCC1.CTRLD = TCC_EVENT_noOut;	// EVACT,EVDLY,EVSEL: 000 0 0000	p.167
	// EVACT[7:5] = 100 event action: restart counter
	// EVDLY[4] = 1	event delay for cascading counters
	// EVSEL[3:0] = event source select
	TCC1.INTCTRLA = TIMER_OVF_INT_low; 	// Interrupt level low, p.125
}


// Timer TCC0 und TCC1 *******************************************

void resetTimer(void) {
	TCC0.CNTL = 0;
	TCC0.CNTH = 0;
	TCC1.CNTL = 0;
	TCC1.CNTH = 0;
}


void enableTimer(void) {	// enable timer TCC0 and TCC1
	TCC0.INTCTRLA |= TIMER_OVF_INT_medium; 	// Interrupt level: 10 medium, p.125, p.168
	TCC1.INTCTRLA |= TIMER_OVF_INT_low; 	// Interrupt level: 01 low, p.125, p.168
}


void disableTimer(void) {	// disable timer TCC0 and TCC1
	TCC0.INTCTRLA = TIMER_OVF_INT_disable; 	// Interrupt level: p.125, p.168
	TCC1.INTCTRLA = TIMER_OVF_INT_disable; 	// Interrupt level: p.125, p.168
}

