#ifndef CLOCKS_H
#define CLOCKS_H


// Event System ******************************************
	// CHxMUX, A-Manual, p.75
	// SysTick verteilen auf dem Event System CH7MUX Kanal 7
	// Event source: Timer C0 Overflow (TCC0_OVF), A-Manual, p.72
//#define EventSource_TCC0_OVF 0b11000000;	// 1100 0 000	EVCHL7 
// #define EventSource_TCC0_OVF 0x00;	// 0000 0 000	EVCHL7 abgeschaltet		##### 

void EVS_config();	// Timer steuert ADC an							#####



/**************************************************************/
// CPU-CLOCKS

// CPU-Clock-Testpin (mgliche Pins sind leider belegt) 
// Port Configuration CLK_EV[1:0] , siehe A-manual Tab.13-7,p.145
/*
#define CLK_PORT   PORTE	// PORT set as output		###

#define CLK_EV_PC7 0x01		// 01 - Clock Ausgang PC7
#define CLK_EV_PD7 0x02		// 10 - Clock Ausgang PD7
#define CLK_EV_PE7 0x03		// 11 - Clock Ausgang PE7
#define CLK_EV_non 0x00		// 00 - Keine Ausgabe
*/
//#define CLK_EV CLK_EV_non	// keine Ausgabe	 		###
// void CPUclk_setpin(void);	// set clockpin


// Clock System and Oscillator 
// Setup CPU-Clock 

// PLLCTRL.PLLFAC p.90
	// 	clksys =  4 * 8MHz = 32MHz: PLLFACTOR = 0x04;
	// 	clksys =  8 * 8MHz = 64MHz: PLLFACTOR = 0x08;
	// 	clksys = 10 * 8MHz = 80MHz: PLLFACTOR = 0x0a;

#define PLLFACTOR 0x04		// 32 MHz, A-manual p.77, p.87, p.91
//#define PLLFACTOR 0x08	// 64 MHz, A-manual p.77, p.87, p.91
//#define PLLFACTOR 0x0a	// 80 MHz, A-manual p.77, p.87, p.91

void CLK_config(void);


/**************************************************************/
// TIMER 

#define	TIMER_OVF_INT_disable	0x00		// TCCx.INTCTRLA
#define	TIMER_OVF_INT_low 		0x01		// TCCx.INTCTRLA
#define	TIMER_OVF_INT_medium	0x02		// TCCx.INTCTRLA
#define	TIMER_OVF_INT_high		0x03		// TCCx.INTCTRLA

// Timer Clock-input is CLKPER or EVSEL p.152
// SysTick Fsys herstellen, dazu CLKPER durch F_TIM teilen

// SysTicks for F_CPU = 32 MHz:

	// 	Tsys	Fsys		F_TIM	F_TIM
	//   s		kHz			dez.	hex.
	// -----------------------------------
	// 125 s	08 kHz		4000	0x0fa0
	// 100 s	10 kHz		3200	0x0c80
	// 41.6 s	24 kHz		1333	0x0535		24006 Hz	41,656 s
	// 31.25 s	32 kHz		1000	0x03e8
	// 30 s	33.3 kHz	960		0x03c0
	// 25 s	40 kHz		800		0x0320
	// 20.8 s	48 kHz		667		0x029b		47976 Hz	20,843 s
	// 20 s	50 kHz		640		0x0280
	// 10.4 s	96 kHz		333		0x014d

	// innerhalb zweier SysTicks mu gesamte Berechnung und 
	// Datenbertragung fertig sein.
	// #define Fsys (F_CPU/F_TIM)	// systick rate in Hz
	// #define Tsys (F_TIM/F_CPU)	// systick duration in sec.

// Init systick 32 MHz --> 8 kHz
#define F_TICK 3987 	// clock divison factor for systick

// Anzahl von Systicks pro Sekunde
#define F_SEC 8000		// bei Systick = 8kHz

// Takte pro Heartbeat
#define F_BEAT 8000		// 32MHz/1024/8000 = 3,90625 Hz ~ 0,256 s

// Init systick 32 MHz --> 10 kHz: (zu schnell)
//#define F_TICK 3190 		// clock divison factor for systick, see tab.

// Systick Testpin *******************************************

#define TICKPORT PORTF		// systick test output port	PF6	###
#define TICKPIN (1<<PIN6)	// pin number for systick 		###

void Systick_setpin(void);
void Systick_config(void);

void toggleTickpin(void);


// Timer0 init ***********************************************
	// input: 	clkcpu 32 MHz
	// output:	SysTick: on event chl 7 
	// 			plus overflow-interrupt (TCC0_OVF)

#define TCC_EVENT_inpClk 0x01 	// CTRLA CLKSEL[3:0] = 0 001, ohne Vorteiler, Source ist clkper

	// start event output on channel no.7 to synchronize ADCs
#define TCC_EVENT_out7	0b10001111	// CTRLD  EVACT,EVDLY,EVSEL: 100 0 1111	p.167	#####

	// EVACT[7:5] = 100 event action select: restart counter
	// EVDLY[4] = 1	event delay for cascading counters
	// EVSEL[3:0] = 1111 event source select 1xxx, 111: event channel 7 is source for ADC and TCC1


// Timer1 init ***********************************************
	// input:	event-channel 7 (SysTick)
	// output: 	half-second clock on event-channel 1
	// 			plus Timer C1 Overflow (TCC1_OVF)

#define TCC_CLKSRC_1024	0x07;	// CTRLA CLKSEL[3:0] = 0 111 (clk/1024 as source)
// #define TCC_EVENT_inp7	0x0f	// CTRLA CLKSEL[3:0] = 1 111 (source is event channel 7)	#####
#define TCC_EVENT_noOut	0x00	// CTRLD  EVACT,EVDLY,EVSEL: 000 0 0000					#####

void Beat_config(void);


// Timer TCC0 und TCC1 ***************************************

void enableTimer(void);		// enable  TCC0 and TCC1
void disableTimer(void);	// disable TCC0 and TCC1
void resetTimer(void);		// reset TCC0 and TCC1


#endif ////////////////////////// end of header /////////////////////////////////
