#ifndef CONTROL_H
#define CONTROL_H



// PMIC #######################################################

// set programmable interrupt controller (PMIC) p.128
//#define PMIC_SET_robin 0b10000111	// 0b1-------: RoundRobin-Scheduling 
#define PMIC_SET_himedlo 0b00000111	// 0b-----xxx: Interrupt enable hi, med and low
void PMIC_init(void);


// Watchdog ###################################################

#define WDT_CEN		0x01		// ATXmega-A-series, S.120 ff.
#define WDT_ENABLE	0x02		// ATXmega-A-series
#define watchdog_reset()	asm("wdr")	// assembler instruction, siehe avr_compiler.h ???

void watchdog_config(void);

// Reset ######################################################

void softReset(void);


// Interaction ################################################

void prepareRec(void);
void sendRec(void);
void sendParlist(void);				// send parameter list to PC
void initPotis(void);
void prepareWeld(void);

void setComPort(uint8_t comreg);
void cmdDecode(uint8_t cmd);


// Schweiregler ##############################################

// ADCA lesen
#define ADC_K1 ADCA_CH0RES
#define ADC_K2 ADCA_CH1RES
#define ADC_K3 ADCA_CH2RES
#define ADC_K4 ADCA_CH3RES

// ADCB lesen
#define ADC_K5 ADCB_CH0RES
#define ADC_K6 ADCB_CH1RES
#define ADC_KU ADCB_CH2RES
#define ADC_KI ADCB_CH3RES

// Schweistrom schalten 
// STOP-Signal (Bypass) wird an Optokoppler geliefert
// nBY	~PC1	Optokoppler-out		 low-active
#define BYPORT 	PORTC			// ###
#define BYPIN 	(1<<PIN1)		// ###

void bypass_config(void);		// Port und Pin Einstellung
void schweissstrom(uint8_t strom);		// strom = {start, stop}, Schweipuls Start/Stop

// 12 Bit ADC: Wertebereich zwischen MIN und MAX: 
#define MAX	0x07ff	// +2047	max 	12 Bit right adjusted
#define MIN	0xf800	// -2048	min 	12 Bit right adjusted


// Schweissregler:
void R0(void);
void R1(void);
void R2(void);
void R3(void);
void R4(void);
void R5(void);
void R6(void);
void R7(void);
void R8(void);
void R9(void);




#endif ////////////////////////// end of header /////////////////////////////////
