#include "../xmega/avr_compiler.h"		// ldt Pins und Ports
#include "dac.h"
#include "global.h"



// DAC ******************************************************************************


// external Reference Voltages: 
// PA0 <= ARA; VREF fr Photodioden
// PB0 <= ARB; VREF fr zweite Verstrkerstufe -> Offset aller Kanle einstellbar

// #define VREF 0x0883	// ~ Referenzspannung 1.65V

// Setup DAC 
// Ziel: Ausgabe von zwei statischen Spannungen 
// an DAC-Outputs 
// 		PB2 ~ DAC0 ~ AREFB und 
// 		PB3 ~ DAC1 ~ AREFA

void DAC_config()	{
	// Erzeugung zweier Referenzspannungen an PB2, PB3 von 1.65V

	// Dual Channel Operation CHSEL[6:5] = 10; 
	// CH0TRG = CH1TRG = 0 (event-auto-trg off)
	DACB.CTRLB = 0x40;	// 

	// Internal 1.00 V Reference: REFSEL[4:3] = 00 -> max. 0.98 Volt output!
	// besser: Referenz AVCC mit REFSEL[4:3] = 01
	// left adjusted (12 Bit): LEFTADJ[0] = 0 (nein)
	DACB.CTRLC = 0b00001000;

	// min. conv.sequence CONTINTVAL[6:4] = 111; 128/192 CPU-clocks
	// REFRESH[3:0] = 1011 ~ 32768 clks
	DACB.TIMCTRL = 0b01111011;	// 1011 ~ 32768 clks

	// Werte einschreiben (nur high-Teil)
	// 0x0800 - 1 = 0x07ff ~ 2047 (1/2)
	// 0x1000 - 1 = 0x0fff ~ 4095 (max)
	// 0x07FF ~ 2047 ~ 1.55V; 0x0FFF ~ 4095
	// Ziel = (1.65V / 1.55V) * 2047 = 0x0883
	// erst low-Byte schreiben, high-Byte bernimmt den Wert

	DACB.CH0DATA = PRL.VB;	// AREFB = ARB = 1.65 Volt
	DACB.CH1DATA = PRL.VA;	// AREFA = ARA = 1.65 Volt



	// DAC aktivieren: bit0 = 1
	// CH1 enable: bit3 = 1 CH1EN auf PORTB3
	// CH0 enable: bit2 = 1 CH0EN auf PORTB2
	DACB.CTRLA = 0b00001101;

	// Calibration registers not used (+/- 0)
}
