#include "../xmega/avr_compiler.h"		// ldt Pins und Ports
//#include <avr/eeprom.h>			// EEMEM, alternativ:
#define EEMEM __attribute__((section(".eeprom")))

#include "eeprom.h"
#include "global.h"
#include "leds.h"

// EEPROM memory ********************************************************************

// Sources:
	// - http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial#EEPROM
	// - XMEGA-A3 p.14, table 7-3
	// - AN1315 #include <xmega/eeprom_driver.h>
	// - Joachim Rath xeeprom.h

// Special:
	// EEPROM_PAGE_SIZE = 32
	// source address ab 0x001000
	// 64 pages a 32 byte, 
			// E2PAGE = ADDR[10:5], 
			// E2BYTE = ADDR[4:0]


void EEPROM_Init(void)
{
 while((NVM.STATUS & NVM_NVMBUSY_bm) == NVM_NVMBUSY_bm);
 NVM.CTRLB |= NVM_EEMAPEN_bm;
}


// EEPROM write ///////////////////////////////////////////////////////////////////

par_t PAREE EEMEM;	// Parameterliste par_t fr EEPROM anlegen, Block mit 2 pages (64 Byte)


void EEPROMwritePage(uint16_t addr) {
	
	// EEPROM addresses:0x001000...
	NVM.ADDR0 = addr & 0xFF;
 	NVM.ADDR1 = addr >> 8 & 0x1F;	// 32 byte-pages, max. 8k
	NVM.ADDR2 = 0x00;

	// EEPROM Atomic Write (Erase&Write) command
	NVM.CMD   = NVM_CMD_ERASE_WRITE_EEPROM_PAGE_gc;
	CPU_CCP   = CCP_IOREG_gc;		// es sind vier Takte frei
	NVM.CTRLA = NVM_CMDEX_bm;		// schafft es der Compiler, oder nicht?
	//NVM_EXEC();	// alternativ in Assembler (Atmel) - don't work
    
	// warten bis fertig (jeweils ca. 11 ms)
	while((NVM.STATUS & NVM_NVMBUSY_bm) == NVM_NVMBUSY_bm);
}


void EEPROMwriteBlock(void *Dst, void *Src, size_t lenge) {
	// void EEPROMwriteBlock(void *Dst, void *Src, size_t lenge) {
	uint16_t idxSrc,idxDst; uint8_t Rest;

 	idxDst = (uint16_t) Dst;
 	idxSrc = (uint16_t) Src;
 	Rest = 0;

 	while(lenge) {
   		*(uint8_t *)((uint16_t) idxDst++ + MAPPED_EEPROM_START) = *(uint8_t*) idxSrc++;
   		Rest = 1;

		if(!(idxDst % EEPROM_PAGE_SIZE)) {		// Page Grenze erreicht?
			EEPROMwritePage(idxDst-1);
			Rest = 0;
			display(off);			// Block fertig Anzeige ###
		}
	  lenge--;
	};

	if(Rest)
	EEPROMwritePage(idxDst-1);
}



void EEPROMsaveParlist(void) {
	// call EEPROMwriteBlock(&dst, &src, 10);
	// EEPROMwriteBlock((uint8_t *) &PAREE, (uint8_t *) &PRL, sizeof(par_t));
	EEPROMwriteBlock((uint8_t *) &PAREE, (uint8_t *) &PRL, sizeof(par_t));
}	


// EEPROM read ///////////////////////////////////////////////////////////////////

#include <string.h>					// memcpy


void *EEPROMreadBlock(void *Dst, void *Src, size_t lenge) {
	// void *EEPROMreadBlock(void *Dst, void *Src, size_t lenge) {
	return(memcpy(Dst,(uint8_t *)(uint16_t)Src + MAPPED_EEPROM_START, lenge));
}


void EEPROMreadParlist() {	
	// call EEPROMreadBlock (&dst, &src, 10);
	// EEPROMreadBlock((uint8_t *) &PRL, (uint8_t *) &PAREE, sizeof(par_t));
	EEPROMreadBlock((uint8_t *) &PRL, (uint8_t *) &PAREE, sizeof(par_t));
}

