#ifndef GLOBAL_H
#define GLOBAL_H

// Mehrfachspeicherreservierung fr globale Variablen verhindern

#ifndef EXTERN
	#define EXTERN
#else
	#define EXTERN extern
#endif


EXTERN char version[48];		// erste Zeile des Hauptprogramms
EXTERN char error[48];			// Fehlermeldung in ionOS.c


/********************************************************************************/

// Globale Strukturen, Funktionen und Variablen

/*********************************************************************************/


// Funktionszeiger //////////////////////////////////////////////////////

typedef void (*VoidFct)(void);
typedef int (*IntFct)(void);
typedef int (*IntFctInt)(int);


// Typumwandlung ////////////////////////////////////////////////////////

typedef union {		// Zugriff auf Bytes aus 2 Byte
   		uint16_t 	w;
		int16_t		i;
   		char  		b[2];
} char2_t;
char2_t b2; 		// Zugriff z.B. mit b2.b[0]; b2.b[1];


typedef union {		// Zugriff auf Bytes aus 4 Byte
		uint32_t  	d;
		uint16_t 	w[2];
   		char  		b[4];
} char4_t;
char4_t b4; 		// Zugriff z.B. mit b4.b[0]; b4.b[3]; b4.w[1] = ...


typedef union {		// Zugriff auf Bytes aus 32 Byte
		uint32_t 	d[8];		// erlaubter Index [0...7]
		uint16_t 	w[16];		// erlaubter Index [0...15]
		uint8_t  	b[32];		// erlaubter Index [0...31]
} char32_t;
char32_t b32; 		// Zugriff z.B. mit b32.b[30]; b32.w[15]; b32.d[5]


// Kommunikation //////////////////////////////////////////////////////////


/*	Interfaces:

	UART 
		rsx		RS485-dux	USARTC0			-> ETH, USB
		sci		ABIC-SCI	USARTE0			-> Anybus: CAN, PB ...

	SPI 
		pot		Potis		Master 	SPIC 	-> Potentiometer
		ram		RAM			Master 	SPID 	-> MRAM 4 Mb
		ssc		ABIC-SSC	Slave 	SPIE 	-> Anybus
*/

typedef struct {	// wechselnde Prozessorregister, gleiche Zugriffsfunktion
	 volatile uint8_t *datareg;		// ptr auf Datenregister  (UART, SPI)	typedef register8_t in iox*.h
	 volatile uint8_t *statusreg;	// ptr auf Statusregister (UART, SPI)
	 volatile uint8_t *controlreg;	// ptr auf Steuerregister (/ENA MRAM-SPI)
	 volatile uint8_t *portreg;		// ptr auf Portregister   (/CS MRAM-SPI)
	 volatile uint8_t rxflag;		// 
	 volatile uint8_t txflag;		// 
} reg_t;

EXTERN  reg_t REG;


// Kommunikation UART In/Out Buffer ///////////////////////////////////////

enum sig_t {		// Fllstand Buffer
	ready = 0, 		// ready, nothing pending
	buffer_full,	// buffer full
	buffer_empty,	// buffer empty
	string_complete // UART-, SPI-buffer
};

EXTERN volatile enum sig_t bufsig;		// UART, SPI: Buffer


// SPI and UART ////////////////////////////////////////////////////

// Receive universal
uint8_t getch(void); 								// Einzelzeichen (byte)
void 	getx(uint8_t *rxbuffer, uint8_t anz); 		// formatiert (array)
void 	getstr(uint8_t *rxbuffer, uint8_t buflen); 	// unformatiert (string)

// Transmit universal
void 	putch(uint8_t byte);				// Zeichen (byte)
void 	putx(uint8_t *array, uint16_t anz);	// formatiert (array)
void 	putstr(char *string);				// unformatiert (string)

// Typwandler u.a. 
void debugOut(uint16_t value);		// debug-Wertausgabe auf aktuellem UART
void i2asc(uint16_t intValue);		// int to ascii converter



// enum Signale  #####################################################

// Kommunikationsquelle //////////////////////////////////

enum com_t {	// communication source/sink
	rs,			// USARTC0 RS485_duplex
	sci,		// USARTE0 ABIC-SCI
	pot,		// SPIC potentiometer
	ram,		// SPID MRAM-interface
	ssc			// SPIE ABIC-SSC
};

EXTERN volatile enum com_t comreg;


// Arbeitsmoden ///////////////////////////////////////////////////////

enum mode_t	{	
	wld,		// weld only, systick on, no records out
	rec,		// weld + record (on fast, duplex RS485), systick on
	com			// communicate, systick off
};

EXTERN volatile enum mode_t mode;	


// Befehlsdekoder /////////////////////////////////////////////////////////

enum cmd_t {	// Befehlssatz, codiert ist der ASCII hex-Wert
	b	= 0x62,	// burn parameter list (save in EEPROM)
	d	= 0x64,	// debug: weld + record, Parameter: uint8_t (Sekunden)
	f	= 0x66,	// factory reset, init default parlist
	i	= 0x69,	// install new parameter list, volatile
	l	= 0x6c,	// list all current parameters
	r 	= 0x72,	// restart
	w 	= 0x77	// weld only, no recording
};				// commands are case sensitive

EXTERN volatile enum cmd_t cmd;	// cmd (global) ist vom Typ command


// LEDs ///////////////////////////////////////////////////////////////////

enum led_t {		// die LEDs sind damit zu schalten:
	off,		// int 0
	green,		// int 1
	yellow,		// int 2
	red,		// int 3
	blue,		// int 5
	toggle		// int 4
};

EXTERN volatile enum led_t leds;	// volatile enum datentyp variable
	// leds ist nun vom Typ led_t
	// Aufrufe z.B. leds=green; display(leds); display(red)


// Datenrecorder Ausgabe /////////////////////////////////////////////////////
	// Achtung: Kommunikation mit Lion nur ber Reihenfolge! Nicht vernderbar!

typedef struct	{		// global (6 Byte + CH*2 byte hex)	 CH = 13: frame = 32 Byte)
						// measured data, output on each systick
	 uint8_t	FS;		// Framesync, identifier for frame synchronisation (G)
	 uint8_t 	ID;		// Parametersatz-Identifier (z.B. 0x48)
	 uint16_t 	TS;		// time stamp, sample (wird mit systick incrementiert)
	 uint8_t	SEC;	// aktuelle Sekunde
	 uint8_t	CH;		// Number of following chls a 16 bit, 0x0d = 13

	 int16_t 	K1; 	// result of 12 bit AD-conversion (a 2 byte, L -> H), right adjusted, little endian
	 int16_t 	K2; 	// signed Wertebereich: von 0xf800 = -2048 bis  0x07ff = 2047; (Theorie ohne Atmel)
	 int16_t 	K3; 
	 int16_t 	K4; 
	 int16_t 	K5; 
	 int16_t 	K6; 
	 int16_t 	KU; 
	 int16_t 	KI;

	 int16_t 	RA; 	// Results of calculations
	 int16_t 	RB; 
	 int16_t 	RC; 
	 int16_t 	RD; 
	 int16_t 	RE; 
} rec_t;

// recs struct physisch anlegen:
EXTERN  rec_t REC;		// Zugriff z.B. ber REC.TS = 0x7890, REC.K1 = 0x0234

// REC-struct initialisieren
void RECinit(void);

// Teststruktur aufbauen mit ASCII 0...9, a...t
void genTestRec(void);			// generiert Teststruktur


// Regelparameter ///////////////////////////////////////////////////////////////
	// Achtung: Kommunikation mit Lion nur ber Reihenfolge! Nicht vernderbar!

typedef struct	{	
	// hex-Parameter im EEPROM 32 x 16 bit = 512 Bit = 64 Byte
	// diese werden nach Reset ins RAM geladen und bei Burn ins EEPROM gebrannt

	 // Einstellungen
	  uint8_t  ID; 		// Ident.-Nr. dieses Parametersatzes
	  uint8_t  SEC;		// zu sendende Sekunden: speziell: PRL.SEC = 0: permanent
	  uint8_t  AO; 		// Autooffset on/off [7...0];  channels [8...1]
	  uint8_t  NE; 		// Negation bits NEG[7...0]; channels [8...1]

	  uint16_t FT; 		// Systicks pro Preview
	  // alt: uint16_t FT; 		// CPU-Clocks pro Systick
	  uint16_t FH; 		// Systicks pro Sekunde
	  uint16_t NA; 		// Nummer des Weld-Algorithmus
	  uint16_t NP; 		// Nummer der Diodenanordnung

	 // Potentiometer
	  uint16_t P1; 		// Gain Potentiometer Channel 1...n
	  uint16_t P2;
	  uint16_t P3; 
	  uint16_t P4; 
	  uint16_t P5; 
	  uint16_t P6; 
	  uint16_t PU; 		// Gain U
	  uint16_t PI; 		// Gain I
	 
	 // ADC Offsets	
	   int16_t O1;		// Offset channel 1
	   int16_t O2;
	   int16_t O3;
	   int16_t O4;
	   int16_t O5;
	   int16_t O6;
	   int16_t OU;		// Offset channel U
	   int16_t OI;		// Offset channel I

	 // Referenzspannungen
	   int16_t VA; 		// DAC-output VREF A -> ARA (right adj)
	   int16_t VB; 		// DAC-output VREF B -> ARB (right adj)

	 // freie Reglereinstellungen
	   int16_t G1;	// gain1
	   int16_t T1; 	// time1	Pausendauer
	   int16_t G2;	// gain2
	   int16_t T2; 	// time2	Pulsverlngerung
	   int16_t G3;	// gain3
	   int16_t T3; 	// time3
	   int16_t TA; 	// Anfangs-Schwellwert fr START welding 	(steuert den Bypass)
	   int16_t TE; 	// End-Schwellwert fr STOP welding 		(steuert den Bypass)

} par_t;

EXTERN  par_t PRL;		// Zugriff z.B. mit PRL.P1

void genPar(void);			// Parametersatz fr factory-reset generieren


// debug-Frame //////////////////////////////////////////////////////////

typedef struct {				// Befehlsstruktur 128 Byte-Format
	par_t		PRL;			// parameter list, EEPROM-saved 
	uint8_t		C;				// command:  d[0xad], l, w, s[par_t]; byte[64]
	uint8_t		D;				// command data:  byte[65]
	uint8_t		SRC[4];			// source address
	uint8_t		DST[4];			// destination address
	uint8_t		MAC[6];			// MAC-Address of Ion
	uint8_t		DAT[48];		// comment field
} frame_t;

EXTERN  frame_t FR;

void genFrame(void);
void genError(void);


// Schweipuls START/STOP ///////////////////////////////////////////////
enum bypass_t {		// Bypass-Signal auf Optokoppler, low-aktiv
	stop  = 0,		// weld stop
	start = 1,		// weld start
};

volatile enum bypass_t strom;


// Strings ##############################################################

// FLASH TEXT ////////////////////////////////////////////////////
/*
typedef struct {
	volatile uint16_t 	ID;			// parameter set identifier
	prog_uint8_t 		CUR[4];		// cursor
	prog_uint8_t 		TXT[];		// flash text with unknown size
} dbTxt_t;

dbTxt_t HLP;	// Help Text (Flash)
*/

// RAM TEXT //////////////////////////////////////////////////////
/*
typedef struct	{				// global 32 Byte (1 EEPROM-page)
	uint16_t ID;		// identifier
	uint8_t T[30];		// comment
} txt_t;

txt_t TXT;			// Zugriff z.B. mit TEXT.ID = 0x0034
*/

// TIME ////////////////////////////////////////////////////////
/*
typedef struct	{					// global
	uint8_t 	SEC[2];		// second
	uint8_t 	MS[2];		// millisec
} time_t;

 time_t TM; 			// Zugriff z.B. TIME.SEC; TIME.MS
*/

// PSITOOLS-CHL ////////////////////////////////////////////////
// unbenutzt, da Kanle in Blcken abgelegt werden -> kein Streamformat
/*
typedef struct {		// PSI-Tools CHL-Format_v3 DA T.Nguyen 1996, S.14
	// Head:
	 uint16_t 	version;	// 1996: 3
	 uint16_t 	chn;		// Anzahl der Kanle
	 uint32_t 	sn;			// Anz. der Samples (Aufzeichnungsdauer)
	 float 		vmin;		// Minimale Amplitude der Kanle
	 float 		vmax;		// Minimale Amplitude der Kanle
	 uint16_t 	rate;		// Samplerate
	 uint16_t 	gain;		// Verstrkung
	 char 		name[10];	// Text vor chnum
	// Kanle werden danach nacheinander abgelegt
	// die Samples folgen als float32
	// Gesamtanzahl der Samples = chn * sn
} psi_t;
*/


// Binary ######################################################

// Leibniz-Funktionen (engl. Boolean functions) 
// Bit-Ops, Beispiel: bit 7 <=> 0x80, bit 3 <=> 0x08
// Makros

// Use Bitposition _bp (BITPOS) 0...7
#define BPSET(PORT,BITPOS) 	(PORT |=  (1<<BITPOS))
#define BPCLR(PORT,BITPOS) 	(PORT &=~ (1<<BITPOS))
#define BPTGL(PORT,BITPOS) 	(PORT ^=  (1<<BITPOS))
#define BPCHK(PORT,BITPOS) 	(PORT  &  (1<<BITPOS))

// Use Bitmask _bm (BYTE)	0xff ... 0x00
#define SET(PORT,BYTE) 		(PORT |= BYTE)		// OR EQ
#define CLR(PORT,BYTE) 		(PORT &=~ BYTE)		// AND EQ NEG
#define TGL(PORT,BYTE) 		(PORT ^= BYTE)		// EXOR EQ
#define CHK(PORT,BYTE) 		(PORT  & BYTE)		// AND


#endif ////////////////////////// end of header /////////////////////////////////
