#include "../xmega/avr_compiler.h"		// ldt Pins und Ports
#include "leds.h"
#include "global.h"


// LED-config ***********************************************

// port/pin assignment for LEDs
	// PD0 ~ LEDR ~ red
	// PD1 ~ LEDG ~ green
	// PD6 ~ LEDB ~ blue
	// LEDs are low active (wired AND) 
	// 0: lightning, 1:dark

/* in .h
#define LEDPORT PORTD					// port
#define LRED (1<<PIN0)					// red LED
#define LGRN (1<<PIN1)					// green LED
#define WIRAND 0b00101000				// wired-AND, OPC = 101, high not driven
*/
// LED-functions ******************************************

void LEDs_OFF() {LEDPORT.OUTSET |= (LRED | LGRN | LBLU);}	// alle off (H)
void LEDs_TGL()	{LEDPORT.OUT ^= LRED; LEDPORT.OUT ^= LGRN; LEDPORT.OUT ^= LBLU;}

void LED_GREEN_OFF()	{LEDPORT.OUTSET |= LGRN;}	// green off, set one, LGRN <-> 0x02 
void LED_GREEN_ON()		{LEDPORT.OUTCLR |= LGRN;}	// green on, set zero
void LED_GREEN_TGL()	{LEDPORT.OUT    ^= LGRN;}	// LED green toggle
void LED_GREEN()		{LEDPORT.OUTCLR |= LGRN; LEDPORT.OUTSET |= LBLU; LEDPORT.OUTSET |= LRED;}	

void LED_RED_OFF()		{LEDPORT.OUTSET |= LRED;}	// red off, set one, LRED <-> 0x01 
void LED_RED_ON()		{LEDPORT.OUTCLR |= LRED;}	// red on, set zero
void LED_RED_TGL()		{LEDPORT.OUT    ^= LRED;}	// LED red toggle
void LED_RED()			{LEDPORT.OUTCLR |= LRED; LEDPORT.OUTSET |= LGRN; LEDPORT.OUTSET |= LBLU;}

void LED_BLUE_OFF()		{LEDPORT.OUTSET |= LBLU;}	// BLUE off, set one, LBLU <-> 0x01 
void LED_BLUE_ON()		{LEDPORT.OUTCLR |= LBLU;}	// BLUE on, set zero
void LED_BLUE_TGL()		{LEDPORT.OUT    ^= LBLU;}	// LED BLUE toggle
void LED_BLUE()			{LEDPORT.OUTCLR |= LBLU; LEDPORT.OUTSET |= LGRN; LEDPORT.OUTSET |= LRED;}

void LED_YELLOW()		{LEDPORT.OUTSET |= LBLU; LEDPORT.OUTCLR |= LRED; LEDPORT.OUTCLR |= LGRN;}

void LEDs_config(void)	{ 
	LEDPORT.DIR |= (LRED | LGRN | LBLU); 		// Set as output
	LEDPORT.PIN0CTRL |= WIRAND; 		// red     Set to wired-AND, OPC = 101	###
	LEDPORT.PIN1CTRL |= WIRAND; 		// green   Set to wired-AND, OPC = 101	###
	LEDPORT.PIN6CTRL |= WIRAND; 		// blue    Set to wired-AND, OPC = 101	###
};


// LED-display ********************************************

void display(enum led_t leds)	{	// enum led_t siehe global.h
	// leds_ Typ ist "enum ledcolor"
	// input: leds: global, changes, if button ENTER pressed
   switch(leds)	{				// global leds
	case off: 		LEDs_OFF(); 					break;
	case green: 	LED_GREEN();				 	break;
	case yellow:	LED_YELLOW(); 			 		break;
	case red: 		LED_RED(); 						break;
	case blue: 		LED_BLUE(); 					break;
	case toggle: 	LEDs_TGL(); 					break;
	default: 		LEDs_OFF();
	}
}



