#ifndef SPI_H
#define SPI_H


// general SPI **************************************************************

// gilt fr alle I/O-Ports PINnCTRL im IO-space:
#define TOTEMPOLE 		0b00000000	// OPC = 000, p.142
#define BUSKEEPER		0b00001000	// OPC = 001
#define PULLDOWN		0b00010000	// OPC = 010, input with pull down
#define PULLUP 			0b00011000	// OPC = 011, input with pull up
#define WIRANDPULL 		0b00111000	// OPC = 111, output wired AND with pullup
#define WIRAND	 		0b00101000	// OPC = 101, output wired AND

// gilt fr alle SPI-Interfaces im IO-space: 
#define nCS				(1<<PIN4)	// SPI /SS Pin
#define MOSI			(1<<PIN5)	// SPI  SI Pin
#define MISO			(1<<PIN6)	// SPI  SO Pin
#define SCK				(1<<PIN7)	// SPI  CLK Pin

// gilt fr alle SPI-Interfaces SPI-Register CTRL:
#define SPI_CLKPER2		0b10000000	// CLKPER/2, A-manual p.233; 
#define SPI_CLKPER4		0b00000000	// CLKPER/4, A-manual p.233; 
#define SPI_CLKPER8		0b10000001	// CLKPER/8, A-manual p.233; 	4MHz
#define SPI_CLKPER16	0b00000001	// CLKPER/16, A-manual p.233; 	2MHz
#define SPI_CLKPER64	0b00000010	// CLKPER/64, A-manual p.233; 	500kHz


// suffers global defines for datareg and statusreg

void 	putc_SPI(uint8_t byte);						// put byte
void 	putw_SPI(uint16_t word);					// put word
void 	putx_SPI(uint8_t *array, uint8_t numof);	// put a number of hex bytes

uint8_t 	getc_SPI(void);							// get byte
uint16_t 	getw_SPI(void);							// get word
void 		getx_SPI(uint8_t *array, uint8_t numof);	// get a number of hex bytes



// SPI circuits and potentiometer ***************************************

#define POTPORT 			PORTC				// I/Os port C				###
#define POTSPI 				SPIC				// SPI port C				###

// ATxmega is SPI-master; 
// MCP-potis are SPI-slaves in daisy-chain config.

	// Signal	Pin		Function
	// nSS		PC4		/CS chip select output to MCP42xxx - chain
	// MOSI		PC5		Master output to MCP42xxx
	// MISO		PC6		Master input from MCP42xxx
	// SCK		PC7		SCK clock to MCP42xxx - chain
	//
	// Poti Microchip MCP42xxx: IC hat zwei Potis
	// die richtige Reihenfolge bestimmt die Ankunft:
	// letztes Poti (K5 und K6) zuerst senden
	// erst command byte, dann data byte, msb first
	// Daisy-Chain Configuration,
	// Timing siehe MCP42xxx Datenblatt, p.19, Fig.5-3
	//
	// MCP42 command byte (register cmd):
	//	0b_xxxx_xx10	fr Poti mit Pins 5,6,7	-> K1,K3,K5
	//	0b_xxxx_xx01 	fr Poti mit Pins 8,9,10 ->	K2,K4,K6
	//	0b_xxxx_xx11	both potis (K12, K34, K56) - not in use.
	//
	//	0b_xx01_xxxx	write data
	//	0b_xx10_xxxx	shutdown
	//
	// Wert 0x00 bedeutet 0 Ohm zwischen Pins 5-6 bzw.9-10



#define POTG		0x11			// Command fr MCP42 "Poti gerade"
#define POTU		0x12			// Command fr MCP42 "Poti ungerade"
#define POTRATE 	SPI_CLKPER16	// A-manual p.232; 
									// Poti max. Rate 5 MHz
void SPI_POTIconfig();
void sendPotiGains();


// SPI for AnyBus-IC ********************************************************
// ABIC-SSC-Interface fr CAN, Profibus, SPI as SLAVE (!)

#define SSCPORT 	PORTE				// SPI IOs auf Port			###
#define SSCSPI		SPIE				// SPI auf Port				###

void SSC_SPI_config(); 					// ABIC SPI-Interface SSC



// ABIC-SSC FIFO buffer *****************************************************
/*
#define FIFOleng 32				// FIFO-lenght in byte (1...255)	###
uint8_t fifo(uint8_t inbyte);	// ABIC-SSC FIFO bereitstellen
*/


#endif ////////////////////////// end of header /////////////////////////////////
