#include "../xmega/avr_compiler.h"		// ldt Pins und Ports
#include <stdint.h>				// uint8_t etc. Std-Integers
#include "uart.h"
#include "global.h"
#include "control.h"
/*
#include <string.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <stdio.h>					// printf und scanf etc.
#include <stdlib.h>
*/

#include <avr/io.h>


// UART general tools ********************************************************
// siehe auch http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial/Der_UART 

/* 	Die globalen Variablen 
		statusreg
		datareg
	sind vorab zu setzen!
*/


// UART-Transmit universell //////////////////////////////////////////////////////

void putc_UART(uint8_t sbyte) {			// fr fast alle UARTs verwendbar, 
	// nutzt globale Variablen fr USART Status- und Datenregister
	// wait until transmit buffer is empty (data register empty flag - DREIF)
	while (!(*(REG.statusreg) & USART_DREIF_bm)) { }	// tx data register empty
	*(REG.datareg) = sbyte;		// write byte to uart
}


void put2c_UART(uint8_t first,uint8_t second) {	// zwei Byte senden
	putc_UART(first);	// 
	putc_UART(second);	// 
}


void putw_UART(uint16_t word) {	// Wort senden, Byteorder: high, low
	uint8_t lowbyte = (uint8_t) word;	// cast von 16 auf 8
	uint8_t highbyte = (uint8_t) (word>>8);	// cast and shift
	putc_UART(highbyte);
	putc_UART(lowbyte);
}


void puti_UART(uint16_t *array, uint16_t anz) {	// big endian, IBM
	while (anz > 0)	{ 					// send hex word array, msb first
		put2c_UART(*array+1, *array); 	// most significant byte first
		anz--; 
		array++;
	}	
}


inline void putx_UART(uint8_t *array, uint16_t anz) {	// little endian, X86
	while (anz > 0)	{ 				// send hex byte array, lsb first
		putc_UART(*array++); 		// least significant byte first
		anz--; 
	}	
	bufsig = ready;
}


void puts_UART(char *string)	{	// sende char null-terminierten string
	while (*string) {				// while string is true
		putc_UART(*string++);
	}
	bufsig = ready;
}


// Null-terminierten String aus dem Flash senden
void putf_UART(const char *flashstring)
{
    while (1)
    {
        unsigned char ch = pgm_read_byte(flashstring);
        flashstring++;
        if ('\0' == ch) break;
        putc_UART(ch);
    }
	bufsig = ready;
}



// UART-Receive universell ////////////////////////////////////////////////

uint8_t getc_UART(void) {	// RXCIF wird gelscht, wenn RX-Buffer leer ist p.249
	while (!(*(REG.statusreg) & USART_RXCIF_bm)) {;} 	// warte bis Zeichen da ###
	return (*(REG.datareg));							// Byte von UART holen
}


inline void getx_UART(uint8_t *rxbuffer, uint8_t buflen) {	// hex, interruptfhig 
	// getx_UART braucht ca. 66 Zyklen. Bei fCPU = 32 MHz und 4 Mbit/s -> 
	// 400 kByte/s stehen aber nur 80 Takte zur Verfgung: Vorsicht!
	uint8_t nextchar;	
	static uint8_t bufnum;		// static: auto-init mit Null(!) hier 0...127

	while (!(*(REG.statusreg) & USART_RXCIF_bm)) {;} 	// warte bis Zeichen da ###
	nextchar = (*(REG.datareg));

	while( bufnum < buflen-1 ) {
		*rxbuffer++ = nextchar;
		bufnum++;
		while (!(*(REG.statusreg) & USART_RXCIF_bm)) {;} 	// warte bis Zeichen da 	###
		nextchar = (*(REG.datareg));
	}
	bufnum = 0;		// fertig
	bufsig = string_complete;
}


void gets_UART(uint8_t *rxbuffer, uint8_t buflen) {	// ASCIIs, interruptfhig 
	// gets_UART braucht ca. 66 Zyklen. Bei fCPU = 32 MHz und 4 Mbit/s -> 
	// 400 kByte/s stehen 80 Takte zur Verfgung: Vorsicht.
	// buflen teilt max. Gre des Buffers mit: Overflow vermeiden
	uint8_t nextchar;	
	static uint8_t bufnum;
					
	while (!(*(REG.statusreg) & USART_RXCIF_bm)) {;} 	// warte bis Zeichen da ###
	nextchar = (*(REG.datareg));


	while(	(nextchar != '\r') && (bufnum < buflen) ) {
		*rxbuffer++ = nextchar;
		bufnum++;
		while (!(*(REG.statusreg) & USART_RXCIF_bm)) {;} 	// warte bis Zeichen da 	###
		nextchar = (*(REG.datareg));
	}
	*rxbuffer = '\r';	// Terminierung abweichend von C und Unix
	bufnum = 0;			// fertig
	bufsig = string_complete;
}




// RS485 UART duplex INIT ********************************************************
// Datenrecorder; schneller Port fr Datenbertragung auf RS485 
// duplex (TX und RX separat)


void RS485_config() {			// UART RS485 duplex highest speed

	// Port einstellen:
	RS_PORT.DIRCLR |= UART_RX;  	// RX als Eingang		
	RS_PORT.OUTSET |= UART_TX;		// TX auf high setzen
	RS_PORT.DIRSET |= UART_TX;  	// TX auf Ausgabe

	// set double clock rate
	RS_UART.CTRLB = UART_CLK2X;

	// Baudrate config, A-manual S.238/254
	RS_UART.BAUDCTRLA = BAUDDCCTRLA;		// BSEL[7:0] 	p.254
	RS_UART.BAUDCTRLB = BAUDDCCTRLB;		// BSCALE[3:0], BSEL[11:8]

	// 8 Bit + 1 Stopbit setzen (ist nicht default!) A-manual S.253 Tab. 21-9
	RS_UART.CTRLC |= ACHT_EINSTOPBIT;	// 8 bit: CHSIZE[2:0] = 011 ~ 8bit

	// enable RX und/oder TX
	RS_UART.CTRLB |= UART_RXEN;			// RX enable
	RS_UART.CTRLB |= UART_TXEN;			// TX enable

	// Start interrupts
	RS_UART.CTRLA |= UART_INTCTL_RX_HI;		// [5:0] enable RXC interrupt only
}



// ********* ABIC UART SCI *******************************************************
// SCI-UART ABIC-connector *******************************************************

//SCI_DE		PE0		// Pin 15	RSIO (I/O) fr Halbduplex. Wenn Null: SSC-Auto-operation
//SCI_nINT		PE1		// Pin 13	ABIC: /INT [BLE] /Interrupt-out or [Boot loader enable inp]
//SCI_TX		PE2	RX	// Pin 19	Xmega input
//SCI_RX		PE3	TX	// Pin 17	ABIC input


void SCI_UART_config(void) { // ABIC-SCI-UART initialisieren (Parameter CTRLA.BSEL)

	// Port E config
	SCIPORT.DIRCLR |= UART_RX;  	// RX als Eingang		
	SCIPORT.OUTSET |= UART_TX;		// TX auf high setzen
	SCIPORT.DIRSET |= UART_TX;  	// TX auf Ausgabe

	// USARTE0 config

	// Baudrate config	BSEL und BSCALE = 0
	// A-manual S.238/254
	SCIUART.BAUDCTRLA |= SCI_BAUDCTRLA;	// BSEL_L
	SCIUART.BAUDCTRLB |= SCI_BAUDCTRLB;	// BSEL_H, BSCALE

	// 8 Bit setzen (ist nicht default) A-manual S.253 Tab. 21-9
	// SCIUART.CTRLC = ACHTBIT;			// 8 bit: CHSIZE[2:0] = 011 ~ 0x03
	// 8 bit und zwei Stopbits setzen (ABIC-default)
	SCIUART.CTRLC |= ACHT_ZWEISTOPBIT;

	// Enable RX und TX pins:
	SCIUART.CTRLB |= UART_TXEN;		// set TX pin enabled
	SCIUART.CTRLB |= UART_RXEN;		// set RX pin enabled

	// Start interrupts p.251:
	SCIUART.CTRLA |= UART_INTCTL_RX_HI;	// [5:0] = RX, TX, DRE (low level interrupts)
	// Achtung: Interrupts mssen auch behandelt werden, sonst RESET
}


void SCI_disable() {			// SSC Auto-Mode einschalten
	// #define SCI_DE (1<<PIN0) // RSIO (I/O) fr Halbduplex. Wenn Null: SSC-Auto-operation
	/* SCI_DE (Input & Output) 
	This signal enables data output on half duplex (e.g. RS-485) networks. 
	Connecting this signal to GND disables the SCI channel alltogether 
	and causes the ABIC-module to initialise automatically (i.e. for 
	stand-alone operation).*/

	SCIPORT.DIR |= SCI_DE;		// Datenrichtung Ausgang
	SCIPORT.OUTCLR |= SCI_DE;	// set zero: nur ABIC-SSC wird benutzt
}

void SCI_enable() {				// SSC Auto-Mode ausschalten
	// #define SCI_DE (1<<PIN0) // RSIO (I/O) fr Halbduplex. Wenn Null: SSC-Auto-operation
	/* SCI_DE (Input & Output) 
	This signal enables data output on half duplex (e.g. RS-485) networks. 
	Connecting this signal to GND disables the SCI channel alltogether 
	and causes the ABIC-module to initialise automatically (i.e. for 
	stand-alone operation).*/

	SCIPORT.DIR |= SCI_DE;		// Datenrichtung Ausgang
	SCIPORT.OUTSET |= SCI_DE;	// set one: enable SCI
}


