#ifndef UART_H
#define UART_H


// RS485 UART duplex **************************************************************

// configs
#define RS_PORT 	PORTC			// I/O port C						###
#define RS_UART 	USARTC0			// UART-C-null						###


// Baudrate
// fXCKmax = fPER/2, p.235

// verfgbare defines:
#define BAUD_DCLK_8Mb	0xdffd		// --> dont work (5 kbaud) 
#define BAUD_DCLK_5Mb	0x0000		//
#define BAUD_DCLK_4Mb	0xe001		//
#define BAUD_DCLK_3Mb	0x9055		
#define BAUD_DCLK_2Mb	0xf003		//
#define BAUD_DCLK_1Mb	0x0004		


// Einstellung der Taktrate RS485 bitte hier:
#define DCLKBAUDRATE	BAUD_DCLK_5Mb		// register16_t

// Automatik-Rechnung:
#define BAUDDCCTRLA	(register8_t)	DCLKBAUDRATE			// Baudcontrol A Register, BSEL[7:0] = 0 	p.254
#define BAUDDCCTRLB	((register8_t)	(DCLKBAUDRATE >> 8))	// Baudcontrol B Register, BSCALE[3:0],   (BSEL[11:8])

void RS485_config();		// RS485 als duplex-UART d.h. RX und TX separat



// ********* ABIC UART *****************************************
// SCI-UART ABIC-connector *************************************

/* The SCI channel supports baudrates from 4.8kbps to 57.6kbps. 
By default, it is 19200 Baud. The module attempts to detect the baud rate 
automatically. This is not part of the Modbus RTU specification, 
and requires a special startup sequence (see flowchart) p.20 Anybus */

#define SCIPORT				PORTE			// Port E					###
#define SCIUART				USARTE0			// Uart E null				###

//SCI_DE		PE0		// Pin 15	RSIO (I/O) fr Halbduplex. Wenn Null: SSC-Auto-operation
//SCI_nINT		PE1		// Pin 13
//SCI_TX		PE2	RX	// Pin 19	Xmega input
//SCI_RX		PE3	TX	// Pin 17	ABIC input


// chip45 Baudrate Calculator in ATXmega-Bibl.,		
// normal speed mode USART_CTRLB_CLK2X=0; fosc = 32000000, 
// BAUDCTLB= BSCALE[3:0] + BSEL[11:8];	BAUDCTRLA = BSEL[7:0]; 
// BSEL[11:8] = 0
/*
	RATE	nibble		BSCALE 		dez.	BSEL
	____________________________________________

   115200	  1101		0xd0		131		0x83
    57600	  1110		0xe0		135		0x87
    38400	  0000		0x00		51		0x33
    19200	  0000		0x00		103		0x67	// SCI default
     9600	  0000		0x00		207		0xcf
*/	
//						CTLB,CTLA
#define BAUD_57600		0xe087
#define BAUD_38400		0x0033
#define BAUD_19200		0x0067			// ABIC default
#define BAUD_9600		0x00cf
// #define BAUD_SCI 19200		// ABIC-SCI UART default baudrate
// #define SCIBSEL 0x67		// SCI-BSEL = (F_CPU/(16*BAUD_SCI)-1) = 103,166... dez ~ 0x67
// #define SCIBSEL (F_CPU/(16 * BAUD_SCI)-1) // = 103,166... dez ~ 0x67

#define BAUDRATE_SCI	BAUD_19200		// register16_t					###

// A-manual S.238/254
#define SCI_BAUDCTRLB	((int8_t)	(BAUDRATE_SCI >> 8))	// MSB
#define SCI_BAUDCTRLA	(int8_t)	BAUDRATE_SCI			// LSB
#define SCI_DE   (1<<PIN0)	// RSIO (I/O) fr Halbduplex. Wenn Null: SSC-Auto-operation
#define SCI_nINT (1<<PIN1)	// ABIC: /INT [BLE] /Interrupt-out or [Bootloader enable inp]


void SCI_UART_config(void);		// ABIC-SCI initialisieren

/* SCI_DE (Input & Output) --- nicht benutzt
This signal enables data output on half duplex (e.g. RS-485) networks. Connecting this signal to
GND disables the SCI channel alltogether and causes the ABIC-module to initialise automatically (i.e.
for stand-alone operation).*/

void SCI_disable();			// SSC Auto-Mode einschalten
void SCI_enable();			// SSC Auto-Mode ausschalten



// Xmega UART General Parameter ***********************************************

// Pins in IO-space
#define UART_RX	(1<<PIN2)		// pin Eingang RX	an allen UARTs des Xmega
#define UART_TX	(1<<PIN3)		// pin Ausgang TX	an allen UARTs des Xmega

// UART CTRLB
#define UART_CLK2X 	(1<<2)				// CTRLB: double clock rate
#define UART_TXEN  	(1<<3)				// CTRLB: Enable Tx
#define UART_RXEN  	(1<<4)				// CTRLB: Enable Rx

// UART CTRLC
#define ACHT_EINSTOPBIT		0b00000011		// CTRLC; 8bit + 1Stop fr RS485, p.252
#define ACHT_ZWEISTOPBIT	0b00001011		// CTRLC; 8bit + 2Stop fr ABIC
//#define ACHTBIT		(1<<PIN1)|(1<<PIN0)	// CTRLC; ist nicht default!
//#define ZWEISTOP		(1<<PIN3)			// Zwei Stopbits

// UART STATUS
#define UART_RXCIF	(1<<7)				// STATUS: Rx interrupt flag
#define UART_TXCIF 	(1<<6)				// STATUS: Tx interrupt flag
#define UART_DREIF 	(1<<5)				// STATUS: data register empty flag
#define UART_FERR 	(1<<4)				// STATUS: frame error
#define UART_BUFOVF	(1<<3)				// STATUS: buffer overflow

// UART interrupt control
//#define UART_INTCTL_LO  0b00010101		// Start RX, TX, DRE Interrupts low
//#define UART_INTCTL_MED 0b00101010		// Start RX, TX, DRE Interrupts medium
//#define UART_INTCTL_HI  0b00111111		// Start RX, TX, DRE Interrupts high
#define UART_INTCTL_RX_LO   0b00010000		// Start RX Interrupts low
#define UART_INTCTL_RX_MED  0b00100000		// Start RX Interrupts low
#define UART_INTCTL_RX_HI   0b00110000		// Start RX Interrupts low



// UART general functions *********************************************

// UART Tx
void putc_UART(uint8_t sbyte);					// put char
void put2c_UART(uint8_t first,uint8_t second);	// put two chars
void putw_UART(uint16_t word);					// put word
void puti_UART(uint16_t *array, uint16_t anz);	// big endian
void putx_UART(uint8_t *array, uint16_t anz);	// put a number of hex bytes
void puts_UART(char *string);					// 0-terminated char string
void putf_UART(const char *flashstring);		// Flash string to UART

// UART Rx
uint8_t getc_UART(void);						// get char
void getx_UART(uint8_t *rxbuffer, uint8_t buflen); 				// hex-Block, interruptfhig 
void gets_UART(uint8_t *rxbuffer, uint8_t buflen);				// ASCII-String, interruptfhig 




#endif ////////////////////////// end of header /////////////////////////////////
